@@ -20,6 +20,7 @@ details. */
 
 wincaps wincap_vista __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:1,
+  mmap_storage_high:0x070000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:true,
@@ -40,11 +41,13 @@ wincaps wincap_vista __attribute__((section (".cygwin_dll_common"), shared)) = {
     has_case_sensitive_dirs:false,
     has_posix_rename_semantics:false,
     no_msv1_0_s4u_logon_in_wow64:true,
+    has_con_24bit_colors:false,
   },
 };
 
 wincaps wincap_7 __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:1,
+  mmap_storage_high:0x070000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:false,
@@ -65,11 +68,13 @@ wincaps wincap_7 __attribute__((section (".cygwin_dll_common"), shared)) = {
     has_case_sensitive_dirs:false,
     has_posix_rename_semantics:false,
     no_msv1_0_s4u_logon_in_wow64:true,
+    has_con_24bit_colors:false,
   },
 };
 
 wincaps wincap_8 __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:2,
+  mmap_storage_high:0x070000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:false,
@@ -90,11 +95,13 @@ wincaps wincap_8 __attribute__((section (".cygwin_dll_common"), shared)) = {
     has_case_sensitive_dirs:false,
     has_posix_rename_semantics:false,
     no_msv1_0_s4u_logon_in_wow64:false,
+    has_con_24bit_colors:false,
   },
 };
 
 wincaps wincap_8_1 __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:2,
+  mmap_storage_high:0x700000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:false,
@@ -115,11 +122,13 @@ wincaps wincap_8_1 __attribute__((section (".cygwin_dll_common"), shared)) = {
     has_case_sensitive_dirs:false,
     has_posix_rename_semantics:false,
     no_msv1_0_s4u_logon_in_wow64:false,
+    has_con_24bit_colors:false,
   },
 };
 
 wincaps  wincap_10_1507 __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:2,
+  mmap_storage_high:0x700000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:false,
@@ -140,36 +149,13 @@ wincaps  wincap_10_1507 __attribute__((section (".cygwin_dll_common"), shared))
     has_case_sensitive_dirs:false,
     has_posix_rename_semantics:false,
     no_msv1_0_s4u_logon_in_wow64:false,
-  },
-};
-
-wincaps wincap_10_1511 __attribute__((section (".cygwin_dll_common"), shared)) = {
-  def_guard_pages:2,
-  {
-    is_server:false,
-    needs_count_in_si_lpres2:false,
-    needs_query_information:false,
-    has_gaa_largeaddress_bug:false,
-    has_broken_alloc_console:true,
-    has_console_logon_sid:true,
-    has_precise_system_time:true,
-    has_microsoft_accounts:true,
-    has_processor_groups:true,
-    has_broken_prefetchvm:false,
-    has_new_pebteb_region:true,
-    has_broken_whoami:false,
-    has_unprivileged_createsymlink:false,
-    has_unbiased_interrupt_time:true,
-    has_precise_interrupt_time:true,
-    has_posix_unlink_semantics:false,
-    has_case_sensitive_dirs:false,
-    has_posix_rename_semantics:false,
-    no_msv1_0_s4u_logon_in_wow64:false,
+    has_con_24bit_colors:false,
   },
 };
 
 wincaps wincap_10_1703 __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:2,
+  mmap_storage_high:0x700000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:false,
@@ -190,11 +176,13 @@ wincaps wincap_10_1703 __attribute__((section (".cygwin_dll_common"), shared)) =
     has_case_sensitive_dirs:false,
     has_posix_rename_semantics:false,
     no_msv1_0_s4u_logon_in_wow64:false,
+    has_con_24bit_colors:true,
   },
 };
 
 wincaps wincap_10_1709 __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:2,
+  mmap_storage_high:0x700000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:false,
@@ -215,11 +203,13 @@ wincaps wincap_10_1709 __attribute__((section (".cygwin_dll_common"), shared)) =
     has_case_sensitive_dirs:false,
     has_posix_rename_semantics:false,
     no_msv1_0_s4u_logon_in_wow64:false,
+    has_con_24bit_colors:true,
   },
 };
 
 wincaps wincap_10_1803 __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:2,
+  mmap_storage_high:0x700000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:false,
@@ -240,11 +230,13 @@ wincaps wincap_10_1803 __attribute__((section (".cygwin_dll_common"), shared)) =
     has_case_sensitive_dirs:true,
     has_posix_rename_semantics:false,
     no_msv1_0_s4u_logon_in_wow64:false,
+    has_con_24bit_colors:true,
   },
 };
 
 wincaps wincap_10_1809 __attribute__((section (".cygwin_dll_common"), shared)) = {
   def_guard_pages:2,
+  mmap_storage_high:0x700000000000LL,
   {
     is_server:false,
     needs_count_in_si_lpres2:false,
@@ -265,6 +257,7 @@ wincaps wincap_10_1809 __attribute__((section (".cygwin_dll_common"), shared)) =
     has_case_sensitive_dirs:true,
     has_posix_rename_semantics:true,
     no_msv1_0_s4u_logon_in_wow64:false,
+    has_con_24bit_colors:true,
   },
 };
 
@@ -301,10 +294,8 @@ wincapc::init ()
 	      caps = &wincap_8;
 	      break;
 	    case 3:
-	      caps = &wincap_8_1;
-	      break;
 	    default:
-	      caps = &wincap_10_1507;
+	      caps = &wincap_8_1;
 	      break;
 	  }
 	break;
@@ -318,8 +309,6 @@ wincapc::init ()
 	  caps = &wincap_10_1709;
 	else if (version.dwBuildNumber >= 15063)
 	  caps = &wincap_10_1703;
-	else if (version.dwBuildNumber >= 10586)
-	  caps = &wincap_10_1511;
 	else
 	  caps = & wincap_10_1507;
     }
